@@ -13,12 +13,13 @@ declare(strict_types=1);
 
 namespace League\Csv;
 
+use InvalidArgumentException;
 use PHPUnit\Framework\Attributes\DataProvider;
 use PHPUnit\Framework\Attributes\Group;
-use PHPUnit\Framework\Attributes\RunInSeparateProcess;
 use PHPUnit\Framework\TestCase;
 use SplFileObject;
 use SplTempFileObject;
+
 use function chr;
 use function function_exists;
 use function ob_get_clean;
@@ -27,6 +28,7 @@ use function strtolower;
 use function tmpfile;
 use function unlink;
 use function xdebug_get_headers;
+
 use const PHP_EOL;
 
 #[Group('csv')]
@@ -95,7 +97,7 @@ EOF;
 
         return [
             'invalid UTF32-BE' => ['', $invalidBOM, ';'],
-            'valid UTF32-BE' => [ByteSequence::BOM_UTF32_BE, $validBOM, ';'],
+            'valid UTF32-BE' => [Bom::Utf32Be->value, $validBOM, ';'],
         ];
     }
 
@@ -106,42 +108,40 @@ EOF;
         clone $this->csv;
     }
 
-    #[RunInSeparateProcess]
     public function testOutputSize(): void
     {
         ob_start();
-        $length = $this->csv->output('test.csv');
+        $length = $this->csv->download('test.csv');
         ob_end_clean();
         self::assertSame(60, $length);
     }
 
     public function testInvalidOutputFile(): void
     {
-        $this->expectException(InvalidArgument::class);
+        $this->expectException(InvalidArgumentException::class);
 
-        $this->csv->output('invalid/file.csv');
+        $this->csv->download('invalid/file.csv');
     }
 
-    #[RunInSeparateProcess]
     public function testOutputHeaders(): void
     {
         if (!function_exists('xdebug_get_headers')) {
             self::markTestSkipped(__METHOD__.' needs the xdebug extension to run');
         }
 
-        $raw_csv = ByteSequence::BOM_UTF8."john,doe,john.doe@example.com\njane,doe,jane.doe@example.com\n";
+        $raw_csv = Bom::Utf8->value."john,doe,john.doe@example.com\njane,doe,jane.doe@example.com\n";
         $csv = Reader::createFromString($raw_csv);
         ob_start();
-        $csv->output('tést.csv');
+        $csv->download('tést.csv');
         ob_end_clean();
         $headers = xdebug_get_headers();
 
         // Due to the variety of ways the xdebug expresses Content-Type of text files,
         // we cannot count on complete string matching.
         self::assertStringContainsString('content-type: text/csv', strtolower($headers[0]));
-        self::assertSame('Content-Transfer-Encoding: binary', $headers[1]);
-        self::assertSame('Content-Description: File Transfer', $headers[2]);
-        self::assertStringContainsString('Content-Disposition: attachment; filename="tst.csv"; filename*=utf-8\'\'t%C3%A9st.csv', $headers[3]);
+        self::assertSame('content-transfer-encoding: binary', strtolower($headers[1]));
+        self::assertSame('content-description: File Transfer', $headers[2]);
+        self::assertStringContainsString('content-disposition: attachment;filename="tst.csv";filename*=UTF-8\'\'t%c3%a9st.csv', $headers[3]);
     }
 
     public function testChunkDoesNotTimeoutAfterReading(): void
@@ -176,9 +176,9 @@ EOF;
 
     public function testChunk(): void
     {
-        $raw_csv = ByteSequence::BOM_UTF8."john,doe,john.doe@example.com\njane,doe,jane.doe@example.com\n";
-        $csv = Reader::createFromString($raw_csv)->setOutputBOM(ByteSequence::BOM_UTF32_BE);
-        $expected = ByteSequence::BOM_UTF32_BE."john,doe,john.doe@example.com\njane,doe,jane.doe@example.com\n";
+        $raw_csv = Bom::Utf8->value."john,doe,john.doe@example.com\njane,doe,jane.doe@example.com\n";
+        $csv = Reader::createFromString($raw_csv)->setOutputBOM(Bom::Utf32Be->value);
+        $expected = Bom::Utf32Be->value."john,doe,john.doe@example.com\njane,doe,jane.doe@example.com\n";
         $res = '';
         foreach ($csv->chunk(32) as $chunk) {
             $res .= $chunk;
@@ -187,6 +187,44 @@ EOF;
         self::assertSame($expected, $res);
     }
 
+    public function testSetOutputBOMTriggersException(): void
+    {
+        $this->expectException(InvalidArgument::class);
+
+        Reader::createFromString()->setOutputBOM('toto');
+    }
+
+    #[DataProvider('provdesValidOutputBomSequences')]
+    public function testSetOutputBOM(string $expected, Bom|string|null $bom): void
+    {
+        self::assertSame(
+            $expected,
+            Reader::createFromString()->setOutputBOM($bom)->getOutputBOM()
+        );
+    }
+
+    public static function provdesValidOutputBomSequences(): array
+    {
+        return [
+            'null' => [
+                'expected' => '',
+                'bom' => null,
+            ],
+            'empty string' => [
+                'expected' => '',
+                'bom' => '',
+            ],
+            'using a BOM string' => [
+                'expected' => Bom::Utf8->value,
+                'bom' => Bom::Utf8->value,
+            ],
+            'using a BOM instance' => [
+                'expected' => Bom::Utf8->value,
+                'bom' => Bom::Utf8,
+            ],
+        ];
+    }
+
     #[DataProvider('provideCsvFilterTestingData')]
     public function testStreamFilterMode(
         AbstractCsv $csv,
@@ -239,8 +277,8 @@ EOF;
     {
         self::assertSame('', $this->csv->getOutputBOM());
 
-        $this->csv->setOutputBOM(ByteSequence::BOM_UTF8);
-        self::assertSame(ByteSequence::BOM_UTF8, $this->csv->getOutputBOM());
+        $this->csv->setOutputBOM(Bom::Utf8->value);
+        self::assertSame(Bom::Utf8->value, $this->csv->getOutputBOM());
 
         $this->csv->setOutputBOM('');
         self::assertSame('', $this->csv->getOutputBOM());
@@ -248,7 +286,7 @@ EOF;
 
     public function testAddBOMSequences(): void
     {
-        $this->csv->setOutputBOM(ByteSequence::BOM_UTF8);
+        $this->csv->setOutputBOM(Bom::Utf8->value);
         $expected = chr(239).chr(187).chr(191).'john,doe,john.doe@example.com'.PHP_EOL
             .'jane,doe,jane.doe@example.com'.PHP_EOL;
         self::assertSame($expected, $this->csv->toString());
@@ -258,10 +296,10 @@ EOF;
     {
         $text = 'john,doe,john.doe@example.com'.PHP_EOL
             .'jane,doe,jane.doe@example.com'.PHP_EOL;
-        $reader = Reader::createFromString(ByteSequence::BOM_UTF32_BE.$text);
-        $reader->setOutputBOM(ByteSequence::BOM_UTF8);
+        $reader = Reader::createFromString(Bom::Utf32Be->value.$text);
+        $reader->setOutputBOM(Bom::Utf8->value);
 
-        self::assertSame(ByteSequence::BOM_UTF8.$text, $reader->toString());
+        self::assertSame(Bom::Utf8->value.$text, $reader->toString());
     }
 
     public function testEscape(): void
@@ -413,26 +451,35 @@ EOF;
         self::assertFalse($reader->isInputBOMIncluded());
     }
 
-    #[RunInSeparateProcess]
+    public function testOutputStripBOM(): void
+    {
+        $raw_csv = Bom::Utf8->value."john,doe,john.doe@example.com\njane,doe,jane.doe@example.com\n";
+        $csv = Reader::createFromString($raw_csv);
+        $csv->setOutputBOM(Bom::Utf16Be->value);
+
+        ob_start();
+        $csv->download();
+        /** @var string $result */
+        $result = ob_get_clean();
+
+        self::assertStringNotContainsString(Bom::Utf8->value, $result);
+        self::assertTrue(str_starts_with($result, Bom::Utf16Be->value));
+    }
+
     public function testOutputDoesNotStripBOM(): void
     {
-        $raw_csv = ByteSequence::BOM_UTF8."john,doe,john.doe@example.com\njane,doe,jane.doe@example.com\n";
+        $raw_csv = Bom::Utf8->value."john,doe,john.doe@example.com\njane,doe,jane.doe@example.com\n";
         $csv = Reader::createFromString($raw_csv);
-        $csv->setOutputBOM(ByteSequence::BOM_UTF16_BE);
-        ob_start();
-        $csv->output();
-        /** @var string $result */
-        $result = ob_get_clean();
-        self::assertStringNotContainsString(ByteSequence::BOM_UTF8, $result);
-        self::assertStringContainsString(ByteSequence::BOM_UTF16_BE, $result);
-
+        $csv->setOutputBOM(Bom::Utf16Be->value);
         $csv->includeInputBOM();
+
         ob_start();
-        $csv->output();
+        $csv->download();
         /** @var string $result */
         $result = ob_get_clean();
-        self::assertStringContainsString(ByteSequence::BOM_UTF16_BE, $result);
-        self::assertStringContainsString(ByteSequence::BOM_UTF8, $result);
-        self::assertTrue(str_starts_with($result, ByteSequence::BOM_UTF16_BE.ByteSequence::BOM_UTF8));
+
+        self::assertStringContainsString(Bom::Utf16Be->value, $result);
+        self::assertStringContainsString(Bom::Utf8->value, $result);
+        self::assertTrue(str_starts_with($result, Bom::Utf16Be->value.Bom::Utf8->value));
     }
 }
