@@ -14,10 +14,11 @@ declare(strict_types=1);
 namespace League\Csv;
 
 use PHPUnit\Framework\Attributes\DataProvider;
+use PHPUnit\Framework\Attributes\DoesNotPerformAssertions;
 use PHPUnit\Framework\Attributes\Group;
-use PHPUnit\Framework\TestCase;
 use SplFileObject;
 use SplTempFileObject;
+
 use function array_keys;
 use function count;
 use function fclose;
@@ -27,7 +28,7 @@ use function json_encode;
 use function unlink;
 
 #[Group('reader')]
-final class ReaderTest extends TestCase
+final class ReaderTest extends TabularDataReaderTestCase
 {
     private Reader $csv;
     private array $expected = [
@@ -38,6 +39,7 @@ final class ReaderTest extends TestCase
     protected function setUp(): void
     {
         $tmp = new SplTempFileObject();
+        $tmp->setCsvControl(escape: '\\');
         foreach ($this->expected as $row) {
             $tmp->fputcsv($row);
         }
@@ -50,14 +52,34 @@ final class ReaderTest extends TestCase
         unset($this->csv);
     }
 
-    public function testCountable(): void
+    protected function tabularData(): TabularDataReader
     {
-        $source = '"parent name","child name","title"
-            "parentA","childA","titleA"';
-        $csv = Reader::createFromString($source);
-        self::assertCount(2, $csv);
-        $csv->setHeaderOffset(0);
-        self::assertCount(1, $csv);
+        $csv = <<<CSV
+date,temperature,place
+2011-01-01,1,Galway
+2011-01-02,-1,Galway
+2011-01-03,0,Galway
+2011-01-01,6,Berkeley
+2011-01-02,8,Berkeley
+2011-01-03,5,Berkeley
+CSV;
+
+        return Reader::createFromString($csv);
+    }
+
+    protected function tabularDataWithHeader(): TabularDataReader
+    {
+        $csv = <<<CSV
+date,temperature,place
+2011-01-01,1,Galway
+2011-01-02,-1,Galway
+2011-01-03,0,Galway
+2011-01-01,6,Berkeley
+2011-01-02,8,Berkeley
+2011-01-03,5,Berkeley
+CSV;
+
+        return Reader::createFromString($csv)->setHeaderOffset(0);
     }
 
     public function testReaderWithEmptyEscapeChar1(): void
@@ -136,6 +158,7 @@ EOF;
         ];
 
         $file = new SplTempFileObject();
+        $file->setCsvControl(escape: '\\');
         foreach ($raw as $row) {
             $file->fputcsv($row);
         }
@@ -185,7 +208,7 @@ EOF;
     {
         $this->expectException(SyntaxError::class);
 
-        [...$this->csv->getRecords(['field1', 2, 'field3'])];
+        [...$this->csv->getRecords(['field1', 2, 'field3'])]; /* @phpstan-ignore-line */
     }
 
     #[DataProvider('validBOMSequences')]
@@ -193,10 +216,10 @@ EOF;
     {
         /** @var resource $fp */
         $fp = fopen('php://temp', 'r+');
-        fputcsv($fp, $record);
+        fputcsv($fp, $record, escape: '');
         $csv = Reader::createFromStream($fp);
         self::assertSame($expected_bom, $csv->getInputBOM());
-        foreach ($csv as $offset => $row) {
+        foreach ($csv as $row) {
             self::assertSame($expected, $row[0]);
         }
         $csv = null;
@@ -208,14 +231,14 @@ EOF;
     {
         return [
             'withBOM' => [
-                [Reader::BOM_UTF16_LE.'john', 'doe', 'john.doe@example.com'],
-                Reader::BOM_UTF16_LE,
+                [Bom::Utf16Le->value.'john', 'doe', 'john.doe@example.com'],
+                Bom::Utf16Le->value,
                 'john',
             ],
             'withDoubleBOM' =>  [
-                [Reader::BOM_UTF16_LE.Reader::BOM_UTF16_LE.'john', 'doe', 'john.doe@example.com'],
-                Reader::BOM_UTF16_LE,
-                Reader::BOM_UTF16_LE.'john',
+                [Bom::Utf16Le->value.Bom::Utf16Le->value.'john', 'doe', 'john.doe@example.com'],
+                Bom::Utf16Le->value,
+                Bom::Utf16Le->value.'john',
             ],
             'withoutBOM' => [
                 ['john', 'doe', 'john.doe@example.com'],
@@ -227,12 +250,12 @@ EOF;
 
     public function testStripBOMWithEnclosure(): void
     {
-        $source = Reader::BOM_UTF8.'"parent name","child name","title"
+        $source = Bom::Utf8->value.'"parent name","child name","title"
             "parentA","childA","titleA"';
         $csv = Reader::createFromString($source);
         $csv->setHeaderOffset(0);
         $expected = ['parent name' => 'parentA', 'child name' => 'childA', 'title' => 'titleA'];
-        foreach ($csv->getRecords() as $offset => $record) {
+        foreach ($csv->getRecords() as $record) {
             self::assertSame($expected, $record);
         }
     }
@@ -251,13 +274,13 @@ EOF;
 
     public function testDisablingBOMStripping(): void
     {
-        $expected_record = [Reader::BOM_UTF16_LE.'john', 'doe', 'john.doe@example.com'];
+        $expected_record = [Bom::Utf16Le->value.'john', 'doe', 'john.doe@example.com'];
         /** @var resource $fp */
         $fp = fopen('php://temp', 'r+');
-        fputcsv($fp, $expected_record);
+        fputcsv($fp, $expected_record, escape: '');
         $csv = Reader::createFromStream($fp);
         $csv->includeInputBOM();
-        self::assertSame(Reader::BOM_UTF16_LE, $csv->getInputBOM());
+        self::assertSame(Bom::Utf16Le->value, $csv->getInputBOM());
         foreach ($csv as $offset => $record) {
             self::assertSame($expected_record, $record);
         }
@@ -313,7 +336,7 @@ EOF;
         /** @var resource $tmp */
         $tmp = fopen('php://temp', 'r+');
         foreach ($this->expected as $row) {
-            fputcsv($tmp, $row);
+            fputcsv($tmp, $row, escape: '');
         }
 
         $csv = Reader::createFromStream($tmp);
@@ -330,8 +353,7 @@ EOF;
     public function testMapRecordsFields(): void
     {
         $keys = ['firstname', 'lastname', 'email'];
-        $res = $this->csv->getRecords($keys);
-        foreach ($res as $record) {
+        foreach ($this->csv->getRecords($keys) as $record) {
             self::assertSame($keys, array_keys($record));
         }
     }
@@ -344,6 +366,7 @@ EOF;
         ];
 
         $tmp = new SplTempFileObject();
+        $tmp->setCsvControl(escape: '\\');
         foreach ($expected as $row) {
             $tmp->fputcsv($row);
         }
@@ -464,7 +487,7 @@ EOF;
 
     public function testRemovingEmptyRecordsWhenBOMStringIsPresent(): void
     {
-        $bom = Reader::BOM_UTF8;
+        $bom = Bom::Utf8->value;
         $text = <<<CSV
 $bom
 column 1,column 2,column 3
@@ -519,7 +542,7 @@ CSV;
 
     public function testGetHeaderThrowsIfTheFirstRecordOnlyContainsBOMString(): void
     {
-        $bom = Reader::BOM_UTF8;
+        $bom = Bom::Utf8->value;
         $text = <<<CSV
 $bom
 column 1,column 2,column 3
@@ -531,4 +554,102 @@ CSV;
         $this->expectException(Exception::class);
         $csv->getHeader();
     }
+
+    public function testOrderBy(): void
+    {
+        $calculated = $this->csv->sorted(fn (array $rowA, array $rowB): int => strcmp($rowA[0], $rowB[0])); /* @phpstan-ignore-line */
+
+        self::assertSame(array_reverse($this->expected), array_values([...$calculated]));
+    }
+
+    public function testOrderByWithEquity(): void
+    {
+        $calculated = $this->csv->sorted(fn (array $rowA, array $rowB): int => strlen($rowA[0]) <=> strlen($rowB[0])); /* @phpstan-ignore-line */
+
+        self::assertSame($this->expected, array_values([...$calculated]));
+    }
+
+    public function testReaderFormatterUsesOffset(): void
+    {
+        $csv = <<<CSV
+FirstName,LastName,Year
+John,Doe,2001
+Jane,Doe,2005
+CSV;
+
+        $reader = Reader::createFromString($csv);
+        $reader->setHeaderOffset(0);
+        self::assertSame([
+            [
+                'FirstName' => 'John',
+                'LastName' => 'Doe',
+                'Year' => '2001',
+            ],
+            [
+                'FirstName' => 'Jane',
+                'LastName' => 'Doe',
+                'Year' => '2005',
+            ],
+        ], [...$reader]);
+
+        $reader->addFormatter(function (array $record): array {
+            $record['Year'] = (int) $record['Year'];
+
+            return $record;
+        });
+
+        self::assertSame([
+            [
+                'FirstName' => 'John',
+                'LastName' => 'Doe',
+                'Year' =>  2001,
+            ],
+            [
+                'FirstName' => 'Jane',
+                'LastName' => 'Doe',
+                'Year' => 2005,
+            ],
+        ], [...$reader]);
+    }
+
+    public function testHeaderMapper(): void
+    {
+        $csv = <<<CSV
+Abel,14,M,2004
+Abiga,6,F,2004
+Aboubacar,8,M,2004
+Aboubakar,6,M,2004
+CSV;
+        $firstRow = [...Reader::createFromString($csv)
+            ->getRecords([3 => 'Year', 0 => 'Firstname', 1 => 'Count'])][0];
+        self::assertSame(['Year' => '2004', 'Firstname' => 'Abel', 'Count' => '14'], $firstRow);
+    }
+
+    public function testHeaderMapperFailsWithInvalidMapper(): void
+    {
+        $csv = <<<CSV
+Abel,14,M,2004
+Abiga,6,F,2004
+Aboubacar,8,M,2004
+Aboubakar,6,M,2004
+CSV;
+        $this->expectException(SyntaxError::class);
+
+        Reader::createFromString($csv)
+            ->getRecords(['Annee' => 'Year', 'Prenom' => 'Firstname', 'Nombre' => 'Count']);
+    }
+
+    #[DoesNotPerformAssertions]
+    public function testStreamWithFiltersDestructsGracefully(): void
+    {
+        /** @var resource $fp */
+        $fp = fopen('php://temp', 'r+');
+        fputcsv($fp, ['abc', '123'], escape: '');
+
+        $csv = Reader::createFromStream($fp);
+        $csv->addStreamFilter('convert.iconv.UTF-8/UTF-16');
+
+        // An explicitly closed file handle makes the stream filter resources invalid
+        fclose($fp);
+    }
 }
