@@ -13,10 +13,13 @@ declare(strict_types=1);
 
 namespace League\Csv;
 
+use RuntimeException;
 use SeekableIterator;
 use SplFileObject;
 use Stringable;
 use TypeError;
+use ValueError;
+
 use function array_keys;
 use function array_walk_recursive;
 use function fclose;
@@ -40,6 +43,7 @@ use function stream_filter_append;
 use function stream_filter_remove;
 use function stream_get_meta_data;
 use function strlen;
+
 use const SEEK_SET;
 
 /**
@@ -54,9 +58,9 @@ final class Stream implements SeekableIterator
     private bool $is_seekable;
     private bool $should_close_stream = false;
     /** @var mixed can be a null, false or a scalar type value. Current iterator value. */
-    private mixed $value;
+    private mixed $value = null;
     /** Current iterator key. */
-    private int $offset;
+    private int $offset = -1;
     /** Flags for the Document. */
     private int $flags = 0;
     private string $delimiter = ',';
@@ -64,6 +68,7 @@ final class Stream implements SeekableIterator
     private string $escape = '\\';
     /** @var array<string, array<resource>> Attached filters. */
     private array $filters = [];
+    private int $maxLength = 0;
 
     /**
      * @param resource $stream stream type resource
@@ -76,10 +81,18 @@ final class Stream implements SeekableIterator
 
     public function __destruct()
     {
-        array_walk_recursive($this->filters, fn ($filter): bool => @stream_filter_remove($filter));
+        array_walk_recursive($this->filters, static function ($filter): void {
+            if (is_resource($filter)) {
+                set_error_handler(fn (int $errno, string $errstr, string $errfile, int $errline) => true);
+                stream_filter_remove($filter);
+                restore_error_handler();
+            }
+        });
 
-        if ($this->should_close_stream && is_resource($this->stream)) {
+        if ($this->should_close_stream) {
+            set_error_handler(fn (int $errno, string $errstr, string $errfile, int $errline) => true);
             fclose($this->stream);
+            restore_error_handler();
         }
 
         unset($this->stream);
@@ -100,6 +113,11 @@ final class Stream implements SeekableIterator
         ];
     }
 
+    public function ftell(): int|false
+    {
+        return ftell($this->stream);
+    }
+
     /**
      * Returns a new instance from a file path.
      *
@@ -119,9 +137,7 @@ final class Stream implements SeekableIterator
         $resource = fopen(...$args);
         restore_error_handler();
 
-        if (!is_resource($resource)) {
-            throw UnavailableStream::dueToPathNotFound($path);
-        }
+        is_resource($resource) || throw UnavailableStream::dueToPathNotFound($path);
 
         $instance = new self($resource);
         $instance->should_close_stream = true;
@@ -146,16 +162,11 @@ final class Stream implements SeekableIterator
 
     public static function createFromResource(mixed $stream): self
     {
-        if (!is_resource($stream)) {
-            throw new TypeError('Argument passed must be a stream resource, '.gettype($stream).' given.');
-        }
-
-        $type = get_resource_type($stream);
-        if ('stream' !== $type) {
-            throw new TypeError('Argument passed must be a stream resource, '.$type.' resource given');
-        }
-
-        return new self($stream);
+        return match (true) {
+            !is_resource($stream) => throw new TypeError('Argument passed must be a stream resource, '.gettype($stream).' given.'),
+            'stream' !== ($type = get_resource_type($stream)) => throw new TypeError('Argument passed must be a stream resource, '.$type.' resource given'),
+            default => new self($stream),
+        };
     }
 
     /**
@@ -175,14 +186,12 @@ final class Stream implements SeekableIterator
      *
      * @throws InvalidArgument if the filter can not be appended
      */
-    public function appendFilter(string $filtername, int $read_write, array $params = null): void
+    public function appendFilter(string $filtername, int $read_write, ?array $params = null): void
     {
         set_error_handler(fn (int $errno, string $errstr, string $errfile, int $errline) => true);
         $res = stream_filter_append($this->stream, $filtername, $read_write, $params ?? []);
         restore_error_handler();
-        if (!is_resource($res)) {
-            throw InvalidArgument::dueToStreamFilterNotFound($filtername);
-        }
+        is_resource($res) || throw InvalidArgument::dueToStreamFilterNotFound($filtername);
 
         $this->filters[$filtername][] = $res;
     }
@@ -281,17 +290,16 @@ final class Stream implements SeekableIterator
      * @see https://www.php.net/manual/en/splfileobject.rewind.php
      *
      * @throws Exception if the stream resource is not seekable
+     * @throws RuntimeException if rewinding the stream fails.
      */
     public function rewind(): void
     {
-        if (!$this->is_seekable) {
-            throw UnavailableFeature::dueToMissingStreamSeekability();
-        }
+        $this->is_seekable || throw UnavailableFeature::dueToMissingStreamSeekability();
+        false !== rewind($this->stream) || throw new RuntimeException('Unable to rewind the document.');
 
-        rewind($this->stream);
         $this->offset = 0;
         $this->value = false;
-        if (0 !== ($this->flags & SplFileObject::READ_AHEAD)) {
+        if (SplFileObject::READ_AHEAD === ($this->flags & SplFileObject::READ_AHEAD)) {
             $this->current();
         }
     }
@@ -303,11 +311,10 @@ final class Stream implements SeekableIterator
      */
     public function valid(): bool
     {
-        if (0 !== ($this->flags & SplFileObject::READ_AHEAD)) {
-            return $this->current() !== false;
-        }
-
-        return !feof($this->stream);
+        return match (true) {
+            SplFileObject::READ_AHEAD === ($this->flags & SplFileObject::READ_AHEAD) => false !== $this->current(),
+            default => !feof($this->stream),
+        };
     }
 
     /**
@@ -321,24 +328,93 @@ final class Stream implements SeekableIterator
             return $this->value;
         }
 
-        $this->value = $this->getCurrentRecord();
+        $this->value = match (true) {
+            SplFileObject::READ_CSV === ($this->flags & SplFileObject::READ_CSV) => $this->getCurrentRecord(),
+            default => $this->getCurrentLine(),
+        };
 
         return $this->value;
     }
 
+    public function fgets(): string|false
+    {
+        $arg = [$this->stream];
+        if (0 < $this->maxLength) {
+            $arg[] = $this->maxLength;
+        }
+        return fgets(...$arg);
+    }
+
+    /**
+     * Sets the maximum length of a line to be read.
+     *
+     * @see https://www.php.net/manual/en/splfileobject.setmaxlinelen.php
+     */
+    public function setMaxLineLen(int $maxLength): void
+    {
+        0 <= $maxLength || throw new ValueError(' Argument #1 ($maxLength) must be greater than or equal to 0');
+
+        $this->maxLength = $maxLength;
+    }
+
+    /**
+     * Gets the maximum line length as set by setMaxLineLen.
+     *
+     * @see https://www.php.net/manual/en/splfileobject.getmaxlinelen.php
+     */
+    public function getMaxLineLen(): int
+    {
+        return $this->maxLength;
+    }
+
+    /**
+     * Tells whether the end of file has been reached.
+     *
+     * @see https://www.php.net/manual/en/splfileobject.eof.php
+     */
+    public function eof(): bool
+    {
+        return feof($this->stream);
+    }
+
     /**
      * Retrieves the current line as a CSV Record.
      */
     private function getCurrentRecord(): array|false
     {
-        $flag = 0 !== ($this->flags & SplFileObject::SKIP_EMPTY);
+        $isEmptyLine = SplFileObject::SKIP_EMPTY === ($this->flags & SplFileObject::SKIP_EMPTY);
         do {
             $ret = fgetcsv($this->stream, 0, $this->delimiter, $this->enclosure, $this->escape);
-        } while ($flag && is_array($ret) && null === $ret[0]);
+        } while ($isEmptyLine && is_array($ret) && null === $ret[0]);
 
         return $ret;
     }
 
+    /**
+     * Retrieves the current line.
+     */
+    private function getCurrentLine(): string|false
+    {
+        $isEmptyLine = SplFileObject::SKIP_EMPTY === ($this->flags & SplFileObject::SKIP_EMPTY);
+        $dropNewLine = SplFileObject::DROP_NEW_LINE === ($this->flags & SplFileObject::DROP_NEW_LINE);
+        $shouldBeIgnored = fn (string|false $line): bool => ($isEmptyLine || $dropNewLine)
+            && (false !== $line && '' === rtrim($line, "\r\n"));
+        $arguments = [$this->stream];
+        if (0 < $this->maxLength) {
+            $arguments[] = $this->maxLength;
+        }
+
+        do {
+            $line = fgets(...$arguments);
+        } while ($shouldBeIgnored($line));
+
+        if ($dropNewLine && false !== $line) {
+            return rtrim($line, "\r\n");
+        }
+
+        return $line;
+    }
+
     /**
      * Seeks to specified line.
      *
@@ -348,9 +424,7 @@ final class Stream implements SeekableIterator
      */
     public function seek(int $offset): void
     {
-        if ($offset < 0) {
-            throw InvalidArgument::dueToInvalidSeekingPosition($offset, __METHOD__);
-        }
+        $offset >= 0 || throw InvalidArgument::dueToInvalidSeekingPosition($offset, __METHOD__);
 
         $this->rewind();
         while ($this->key() !== $offset && $this->valid()) {
@@ -380,7 +454,7 @@ final class Stream implements SeekableIterator
      *
      * @see https://www.php.net/manual/en/splfileobject.fread.php
      *
-     * @param int<0, max> $length The number of bytes to read
+     * @param int<1, max> $length The number of bytes to read
      */
     public function fread(int $length): string|false
     {
@@ -396,11 +470,25 @@ final class Stream implements SeekableIterator
      */
     public function fseek(int $offset, int $whence = SEEK_SET): int
     {
-        if (!$this->is_seekable) {
-            throw UnavailableFeature::dueToMissingStreamSeekability();
+        return match (true) {
+            !$this->is_seekable => throw UnavailableFeature::dueToMissingStreamSeekability(),
+            default => fseek($this->stream, $offset, $whence),
+        };
+    }
+
+    /**
+     * Write to stream.
+     *
+     * @see http://php.net/manual/en/SplFileObject.fwrite.php
+     */
+    public function fwrite(string $str, ?int $length = null): int|false
+    {
+        $args = [$this->stream, $str];
+        if (null !== $length) {
+            $args[] = $length;
         }
 
-        return fseek($this->stream, $offset, $whence);
+        return fwrite(...$args);
     }
 
     /**
@@ -412,4 +500,19 @@ final class Stream implements SeekableIterator
     {
         return fflush($this->stream);
     }
+
+    /**
+     * Gets file size.
+     *
+     * @see https://www.php.net/manual/en/splfileinfo.getsize.php
+     */
+    public function getSize(): int|false
+    {
+        return fstat($this->stream)['size'] ?? false;
+    }
+
+    public function getContents(?int $length = null, int $offset = -1): string|false
+    {
+        return stream_get_contents($this->stream, $length, $offset);
+    }
 }
