@@ -19,6 +19,7 @@ use PHPUnit\Framework\Attributes\Group;
 use PHPUnit\Framework\TestCase;
 use SplFileObject;
 use SplTempFileObject;
+
 use function array_map;
 use function fclose;
 use function fopen;
@@ -37,8 +38,8 @@ final class WriterTest extends TestCase
     protected function tearDown(): void
     {
         $csv = new SplFileObject(__DIR__.'/../test_files/foo.csv', 'w');
-        $csv->setCsvControl();
-        $csv->fputcsv(['john', 'doe', 'john.doe@example.com'], ',', '"');
+        $csv->setCsvControl(escape: '');
+        $csv->fputcsv(['john', 'doe', 'john.doe@example.com']);
         unset($this->csv);
     }
 
@@ -92,6 +93,10 @@ final class WriterTest extends TestCase
     {
         $csv = Writer::createFromPath(__DIR__.'/../test_files/foo.csv', 'a+');
         $csv->insertOne(['jane', 'doe', 'jane.doe@example.com']);
+
+        self::assertFalse($csv->encloseAll());
+        self::assertFalse($csv->encloseNone());
+        self::assertTrue($csv->encloseNecessary());
         self::assertStringContainsString('jane,doe,jane.doe@example.com', $csv->toString());
     }
 
@@ -158,13 +163,38 @@ final class WriterTest extends TestCase
         /** @var resource $resource */
         $resource = tmpfile();
         $csv = Writer::createFromStream($resource);
-        self::assertSame("\n", $csv->getNewline());
-        $csv->setNewline("\r\n");
+        self::assertSame("\n", $csv->getEndOfLine());
+        $csv->setEndOfLine("\r\n");
         $csv->insertOne(['jane', 'doe']);
         self::assertSame("jane,doe\r\n", $csv->toString());
         $csv = null;
     }
 
+    public function testForceEnclosure(): void
+    {
+        $collection = [
+            [1, 2],
+            ['value 2-0', 'value 2-1'],
+            ['to"to', 'foo\"bar'],
+        ];
+
+        $writer = Writer::createFromString();
+        self::assertFalse($writer->encloseAll());
+
+        $writer->forceEnclosure();
+        self::assertTrue($writer->encloseAll());
+
+        $writer->insertAll($collection);
+        $csv = $writer->toString();
+
+        self::assertStringContainsString('"1","2"'."\n", $csv);
+        self::assertStringContainsString('"value 2-0","value 2-1"'."\n", $csv);
+        self::assertStringContainsString('"to""to","foo\"bar"'."\n", $csv);
+
+        $writer->relaxEnclosure();
+        self::assertFalse($writer->encloseAll());
+    }
+
     public function testAddValidationRules(): void
     {
         $this->expectException(CannotInsertRecord::class);
@@ -183,7 +213,7 @@ final class WriterTest extends TestCase
     {
         $this->expectException(UnavailableStream::class);
         $writer = Writer::createFromPath('php://null', 'w');
-        $writer->setNewline("\r\n");
+        $writer->setEndOfLine("\r\n");
         $writer->insertOne(['foo', 'bar']);
     }
 
@@ -197,7 +227,7 @@ final class WriterTest extends TestCase
     {
         foreach (["\r\n", "\n", "\r"] as $eol) {
             $csv = Writer::createFromString();
-            $csv->setNewline($eol);
+            $csv->setEndOfLine($eol);
             $csv->setEscape('');
             $csv->insertOne($record);
             self::assertSame($expected.$eol, $csv->toString());
@@ -249,4 +279,72 @@ final class WriterTest extends TestCase
             ],
         ];
     }
+
+    public function testEncloseAll(): void
+    {
+        /**
+         * @see https://en.wikipedia.org/wiki/Comma-separated_values#Example
+         */
+        $records = [
+            ['Year', 'Make', 'Model', 'Description', 'Price'],
+            [1997, 'Ford', 'E350', 'ac,abs,moon', '3000.00'],
+            [1999, 'Chevy', 'Venture "Extended Edition"', null, '4900.00'],
+            [1999, 'Chevy', 'Venture "Extended Edition, Very Large"', null, '5000.00'],
+            [1996, 'Jeep', 'Grand Cherokee', 'MUST SELL!
+            air, moon roof, loaded', '4799.00'],
+        ];
+
+        $csv = Writer::createFromString();
+        $csv->setDelimiter('|');
+        $csv->forceEnclosure();
+        $csv->insertAll($records);
+
+        $expected = <<<CSV
+"Year"|"Make"|"Model"|"Description"|"Price"
+"1997"|"Ford"|"E350"|"ac,abs,moon"|"3000.00"
+"1999"|"Chevy"|"Venture ""Extended Edition"""|""|"4900.00"
+"1999"|"Chevy"|"Venture ""Extended Edition, Very Large"""|""|"5000.00"
+"1996"|"Jeep"|"Grand Cherokee"|"MUST SELL!
+            air, moon roof, loaded"|"4799.00"
+CSV;
+
+        self::assertTrue($csv->encloseAll());
+        self::assertFalse($csv->encloseNone());
+        self::assertFalse($csv->encloseNecessary());
+        self::assertStringContainsString($expected, $csv->toString());
+    }
+
+    public function testEncloseNothing(): void
+    {
+        /**
+         * @see https://en.wikipedia.org/wiki/Comma-separated_values#Example
+         */
+        $records = [
+            ['Year', 'Make', 'Model', 'Description', 'Price'],
+            [1997, 'Ford', 'E350', 'ac,abs,moon', '3000.00'],
+            [1999, 'Chevy', 'Venture "Extended Edition"', null, '4900.00'],
+            [1999, 'Chevy', 'Venture "Extended Edition, Very Large"', null, '5000.00'],
+            [1996, 'Jeep', 'Grand Cherokee', 'MUST SELL!
+            air, moon roof, loaded', '4799.00'],
+        ];
+
+        $csv = Writer::createFromString();
+        $csv->setDelimiter('|');
+        $csv->noEnclosure();
+        $csv->insertAll($records);
+
+        $expected = <<<CSV
+Year|Make|Model|Description|Price
+1997|Ford|E350|ac,abs,moon|3000.00
+1999|Chevy|Venture "Extended Edition"||4900.00
+1999|Chevy|Venture "Extended Edition, Very Large"||5000.00
+1996|Jeep|Grand Cherokee|MUST SELL!
+            air, moon roof, loaded|4799.00
+CSV;
+
+        self::assertFalse($csv->encloseAll());
+        self::assertTrue($csv->encloseNone());
+        self::assertFalse($csv->encloseNecessary());
+        self::assertStringContainsString($expected, $csv->toString());
+    }
 }
