@@ -19,6 +19,8 @@ use DOMException;
 use PHPUnit\Framework\Attributes\Group;
 use PHPUnit\Framework\TestCase;
 
+use function array_map;
+
 #[Group('converter')]
 final class XMLConverterTest extends TestCase
 {
@@ -97,4 +99,48 @@ final class XMLConverterTest extends TestCase
         self::assertCount(0, $doc->childNodes);
         self::assertCount(5, $element->childNodes);
     }
+
+    public function testDownload(): void
+    {
+        if (!function_exists('xdebug_get_headers')) {
+            self::markTestSkipped(__METHOD__.' needs the xdebug extension to run');
+        }
+
+        ob_start();
+        XMLConverter::create()->fieldElement('cell', 'name')->download([['foo' => 'bar']], 'foobar.xml');
+        $output = ob_get_clean();
+        $headers = xdebug_get_headers();
+
+        $xml = '<?xml version="1.0" encoding="utf-8"?>'."\n".'<csv><row><cell name="foo">bar</cell></row></csv>'."\n";
+
+        self::assertStringContainsString('content-type: application/xml', strtolower($headers[0]));
+        self::assertSame('content-transfer-encoding: binary', strtolower($headers[1]));
+        self::assertSame('content-description: File Transfer', $headers[2]);
+        self::assertStringContainsString('content-disposition: attachment;filename="foobar.xml"', $headers[3]);
+        self::assertSame($xml, $output);
+    }
+
+    public function testToXMLWithFormatter(): void
+    {
+        $csv = Reader::createFromPath(__DIR__.'/../test_files/prenoms.csv', 'r')
+            ->setDelimiter(';')
+            ->setHeaderOffset(0)
+        ;
+
+        $stmt = Statement::create()
+            ->offset(3)
+            ->limit(5)
+        ;
+
+        $records = $stmt->process($csv);
+
+        $converter = XMLConverter::create()
+            ->rootElement('csv')
+            ->recordElement('record', 'offset')
+            ->fieldElement('field', 'name')
+            ->formatter(fn (array $record, int|string $key): array => array_map(strtoupper(...), $record));
+        ;
+
+        self::assertStringContainsString('ABEL', (string) $converter->convert($records)->saveXML());
+    }
 }
